import 'dart:io';
import 'package:flutter/services.dart';
import 'constants.dart';
import 'ad_base.dart';
import 'adfurikun_plugin.dart';

enum AdfurikunInterstitialCallback {
  onPrepareSuccess,
  onPrepareFailure,
  onStartPlaying,
  onFinishedPlaying,
  onAdClose,
  onFailedPlaying
}

typedef AdfurikunInterstitialListener(
    AdfurikunInterstitialCallback? callback,
    String? appId,
    String? adNetworkKey,
    int? errorCode);

class AdfurikunInterstitial extends AdfurikunAdBase {
  AdfurikunInterstitialListener? listener;

  Map<String, AdfurikunInterstitialCallback> _callbackMap = {
    CALLBACK_ON_PREPARE_SUCCESS: AdfurikunInterstitialCallback.onPrepareSuccess,
    CALLBACK_ON_PREPARE_FAILURE: AdfurikunInterstitialCallback.onPrepareFailure,
    CALLBACK_ON_START_PLAYING: AdfurikunInterstitialCallback.onStartPlaying,
    CALLBACK_ON_FINISHED_PLAYING: AdfurikunInterstitialCallback.onFinishedPlaying,
    CALLBACK_ON_AD_CLOSE: AdfurikunInterstitialCallback.onAdClose,
    CALLBACK_ON_FAILED_PLAYING: AdfurikunInterstitialCallback.onFailedPlaying
  };

  AdfurikunInterstitial(String appId) : super(appId, AD_TYPE_INTERSTITIAL) {
    AdfurikunPlugin.invoke(method(METHOD_INIT), args());
  }

  void load() {
    AdfurikunPlugin.invoke(method(METHOD_LOAD), args());
  }

  void play() {
    AdfurikunPlugin.invoke(method(METHOD_PLAY), args());
  }

  Future<bool> isPrepared() async {
    return await AdfurikunPlugin.invokeReturnBool(method(METHOD_IS_PREPARED), args());
  }

  void resume() {
    if (Platform.isAndroid) {
      AdfurikunPlugin.invoke(method(METHOD_RESUME), args());
    }
  }

  void pause() {
    if (Platform.isAndroid) {
      AdfurikunPlugin.invoke(method(METHOD_PAUSE), args());
    }
  }

  @override
  void destroy() {
    super.destroy();
    AdfurikunPlugin.invoke(method(METHOD_DESTROY), args());
  }

  @override
  void notifyListener(MethodCall call) {
    if (listener != null) {
      AdfurikunInterstitialCallback? callback = _callbackMap[call.method];
      String? appId = call.arguments[KEY_APP_ID];
      String? adNetworkKey = call.arguments[KEY_AD_NETWORK_KEY];
      int? errorCode = call.arguments[KEY_ERROR_CODE];
      listener!(callback, appId, adNetworkKey, errorCode);
    } else {
      print('AdfurikunInterstitialListener is null');
    }
  }
}
